#include "database-operation.h"
#include "utils.h"
#include <QSqlQuery>
#include <QSqlError>
#include <QDate>
#include <QFile>
#include <QRect>
#include <QStandardPaths>
#include <QDebug>

dataBaseOperation::dataBaseOperation(QObject *parent) : QObject(parent), idleItemId(0)
{
    if (!QSqlDatabase::drivers().contains("QSQLITE")) {
        qDebug() << "Unable to load database, this demo needs the SQLITE3 driver ! ! !";
        return;
    }

    QString dbPath = QStandardPaths::standardLocations(QStandardPaths::ConfigLocation)[0] + "/ukui/appList.db";

    // 创建一个数据库连接，指定数据库驱动
    db = QSqlDatabase::addDatabase("QSQLITE");
    db.setDatabaseName(dbPath);
}

dataBaseOperation::~dataBaseOperation()
{
    db.close();
}

bool dataBaseOperation::openDataBase()
{
    if (!db.open()) {
        qDebug() << db.lastError().text();
        return false;
    }
    else {
        if (!isTableExist("itemlist")) {
            createDefaultTable();
            createItemsData();
        }
        qDebug() << "DataBase Tables: " << db.tables();
        return true;
    }
}

void dataBaseOperation::createDefaultTable()
{
    QSqlQuery sqlQuery;
    if (!sqlQuery.exec(QLatin1String("CREATE TABLE itemlist (id integer primary key, itemId integer, desktopName varchar, name varchar, iconName varchar, type integer, placeholder varchar, categories varchar)")))
        qDebug() << sqlQuery.lastError().text();
    if (!sqlQuery.exec(QLatin1String("CREATE TABLE sets (id integer primary key, setId integer, sequence varchar, scrollable integer)")))
        qDebug() << sqlQuery.lastError().text();
    if (!sqlQuery.exec(QLatin1String("CREATE TABLE page (id integer primary key, pageId integer, sequence varchar)")))
        qDebug() << sqlQuery.lastError().text();
}

void dataBaseOperation::createItemsData()
{
    QString desktopName("widgetPlugin1");
    QString sql = "SELECT * FROM itemlist WHERE desktopName=:desktopName";
    QSqlQuery sqlQuery;
    sqlQuery.prepare(sql);
    sqlQuery.bindValue(":desktopName", desktopName);
    sqlQuery.exec();
    while(sqlQuery.next())
    {
        quint32 id = 0;
        id = sqlQuery.value(0).toUInt();
        if(id > 0)
            return;
    }

//    addItem(getIdleItemID(), QLatin1String("widgetPlugin 1"), QString("登录"), QLatin1String(""), 4, {4,2});
//    addItem(getIdleItemID(), "container1", "翻页分组", "", 3, {2,2});
//    addItem(getIdleItemID(), "container2", "滚动分组", "", 3, {2,2});
//    addItem(getIdleItemID(), QLatin1String("/usr/share/applications/peony.desktop"), QString("文件管理器"),
//                QLatin1String("system-file-manager.png"), 1, {1,1}, {9});
//    addItem(getIdleItemID(), QLatin1String("/usr/share/applications/kylin-video.desktop"), QString("影音"),
//                QLatin1String("kylin-video.png"), 1, {1,1}, {7});
//    QList<quint32> pageList = {1,2,3};
//    addFlipSet(2, {{4}});
//    addScrollSet(3, {5});
//    addPage(1, pageList);
    addItem(0, QLatin1String("taskbar"), "taskbar", "", 3, {1, 1});
    addScrollSet(0, {});
}

void dataBaseOperation::updatePageTable()
{
}


bool dataBaseOperation::isTableExist(const QString &tableName)
{
    if(db.tables().contains(tableName, Qt::CaseInsensitive))
        return true;
    else
        return false;
}

quint32 dataBaseOperation::getIdleItemID()
{
    idleItemId = 0;
    QVector<quint32> itemIdList;
    QSqlQuery sqlQuery;
    sqlQuery.exec("SELECT * FROM itemlist");
    if (!sqlQuery.exec())
        qDebug() << sqlQuery.lastError();
    else
    {
        while (sqlQuery.next())
        {
            quint32 itemId = sqlQuery.value(1).toUInt();
            itemIdList.append(itemId);
        }
    }
    qSort(itemIdList);
    for (int var = 0; var < itemIdList.size(); ++var) {
        if (itemIdList[var] != var) {
            idleItemId = var;
            break;
        }
    }
    if (!idleItemId)
        idleItemId = itemIdList.size();
    qDebug() << "idleItemId =" << idleItemId;
    return idleItemId;
}

QMap<quint32, UKUI::BaseItem *> dataBaseOperation::queryAllItemData()
{
    QMap<quint32, UKUI::BaseItem *> itemList;
    QSqlQuery sqlQuery;
    sqlQuery.exec("SELECT * FROM itemlist");
    if(!sqlQuery.exec())
        qDebug() << sqlQuery.lastError();
    else
    {
        while(sqlQuery.next())
        {
            quint32 id = sqlQuery.value(1).toUInt();
            QString desktopName = sqlQuery.value(2).toString();
            QString name = sqlQuery.value(3).toString();
            QString iconName = sqlQuery.value(4).toString();
            int type = sqlQuery.value(5).toInt();
            QStringList list = sqlQuery.value(6).toString().split(',');
            QString x_placeholder = list.at(0);
            QString y_placeholder = list.at(1);
            QList<int> placeHolder = {x_placeholder.toInt(), y_placeholder.toInt()};
            QStringList categories = sqlQuery.value(7).toString().split(',');
            QList<int> category_list;
            foreach (auto category, categories) {
                category_list.append(category.toInt());
            }
//            qDebug() << QString("desktopName:%1    name:%2    id:%3    iconName:%4    type:%5").arg(desktopName).arg(name).arg(id).arg(iconName).arg(type)
//                     << "placeHolder:" << placeHolder;
//            QString absoluteIconName = getAbsoluteIconPath(iconName);
            switch (type) {
            case 1:
                itemList[id] = new UKUI::IconItem(desktopName, name, id, iconName, category_list);
                break;
            case 2:
                itemList[id] = new UKUI::GroupItem(name, id);
                break;
            case 3:
                itemList[id] = new UKUI::ContainerItem(name, id, placeHolder);
                break;
            case 4:
                itemList[id] = new UKUI::Widget(name, id, placeHolder);
                break;
            default:
                break;
            }
        }
    }
    return itemList;
}

QMap<quint32, QList<QList<quint32>>> dataBaseOperation::queryAllFlipSetData()
{
    QMap<quint32, QList<QList<quint32>>> allSets;
    QSqlQuery sqlQuery;
    sqlQuery.exec("SELECT * FROM sets");
    if(!sqlQuery.exec())
        qDebug() << sqlQuery.lastError();
    else
    {
        while(sqlQuery.next())
        {
            if (sqlQuery.value(3).toInt() == 0) {
                int setId = sqlQuery.value(1).toInt();
                QStringList sequence = sqlQuery.value(2).toString().split(';');
                for (int i = 0; i < sequence.size(); ++i) {
                    QStringList setPageItem = sequence.at(i).split(",");
                    QList<quint32> itemList;
                    for (int j = 0; j < setPageItem.size(); ++j) {
                        itemList.append(setPageItem.at(j).toUInt());
                    }
                    allSets[setId].append(itemList);
                }
            }
        }
    }
//    qDebug() << __func__ << "Sets:" << allSets;
    return allSets;
}

QMap<quint32, QList<quint32>> dataBaseOperation::queryAllScrollSetData()
{
    QMap<quint32, QList<quint32>> allSets;
    QSqlQuery sqlQuery;
    sqlQuery.exec("SELECT * FROM sets");
    if(!sqlQuery.exec())
        qDebug() << sqlQuery.lastError();
    else
    {
        while(sqlQuery.next())
        {
            if (sqlQuery.value(3).toInt() == 1)
            {
                int setId = sqlQuery.value(1).toInt();
                QString sequence = sqlQuery.value(2).toString();
                if (sequence.isEmpty())
                {
                    allSets[setId];
                    continue;
                }
                 else {
                    QStringList setAllItem = sequence.split(",");
                    foreach (auto item, setAllItem) {
                        allSets[setId].append(item.toUInt());
                    }
                }
            }
        }
    }
    return allSets;
}

QMap<quint32, QList<quint32>> dataBaseOperation::queryAllPageData()
{
    QMap<quint32, QList<quint32>> allPage;
    QSqlQuery sqlQuery;
    sqlQuery.exec("SELECT * FROM page");
    if(!sqlQuery.exec())
        qDebug() << sqlQuery.lastError();
    else
    {
        while(sqlQuery.next())
        {
            int setId = sqlQuery.value(1).toInt();
            QStringList sequence = sqlQuery.value(2).toString().split(',');
            for (int i = 0; i < sequence.size(); ++i) {
                QString itemId = sequence.at(i);
                allPage[setId].append(itemId.toUInt());
            }
        }
    }
//    qDebug() << __func__ << "Page:" << allPage;
    return allPage;
}

void dataBaseOperation::queryItemDataFromID(int id)
{
    QString sql = "SELECT * FROM itemlist WHERE id=:id";
    QSqlQuery sqlQuery;
    sqlQuery.prepare(sql);
    sqlQuery.bindValue(":id", id);
    sqlQuery.exec();
    while(sqlQuery.next())
    {
        QString desktopName = sqlQuery.value(2).toString();
        QString name = sqlQuery.value(3).toString();
        QString iconName = sqlQuery.value(4).toString();
        int type = sqlQuery.value(5).toInt();
        int parent_id = sqlQuery.value(10).toInt();
        int page = sqlQuery.value(11).toInt();
        QRect geometry = QRect(sqlQuery.value(6).toInt(), sqlQuery.value(7).toInt(), sqlQuery.value(8).toInt(), sqlQuery.value(9).toInt());
        qDebug() << endl << QString("id:%1    name:%2    type:%3    parent_id:%4    page:%5    desktopName:%6    iconName:%7").arg(id).arg(name).arg(type).arg(parent_id).arg(page).arg(desktopName).arg(iconName)
                 << "    geometry =" << geometry << endl;
    }
}

QList<quint32> dataBaseOperation::queryPageDataFromID(int pageId)
{
    QList<quint32> itemList;
    QString sql = "SELECT * FROM page WHERE pageId=:pageId";
    QSqlQuery sqlQuery;
    sqlQuery.prepare(sql);
    sqlQuery.bindValue(":pageId", pageId);
    sqlQuery.exec();
    while(sqlQuery.next())
    {
        QStringList itemListStr = sqlQuery.value(2).toString().split(',');
        if(itemListStr.size() == 1 && itemListStr.at(0).isEmpty())
            return itemList;
        for(int i = 0; i < itemListStr.size(); i++) {
            itemList.append(static_cast<QString>(itemListStr.at(i)).toUInt());
        }
        return itemList;
    }
    return itemList;
}

quint32 dataBaseOperation::queryIdFromDesktopName(QString desktopName)
{
    QString sql = "SELECT * FROM itemlist WHERE desktopName=:desktopName";
    QSqlQuery sqlQuery;
    sqlQuery.prepare(sql);
    sqlQuery.bindValue(":desktopName", desktopName);
    sqlQuery.exec();
    while(sqlQuery.next())
    {
        quint32 id = sqlQuery.value(1).toUInt();
//        qDebug() << __func__ << "id =" << id;
        return id;
    }
    return 0;
}

QVariant dataBaseOperation::addItem(quint32 itemId, const QString &desktopName, const QString &name, const QString &iconName, const QVariant &type,
                                const QList<quint32> &_placeholder, const QList<int> &_categories)
{
    QString placeholder = UKUI::fromQListUIntToString(_placeholder);
    QString categories = UKUI::fromQListIntToString(_categories);
    QSqlQuery sqlQuery;
    if (!sqlQuery.prepare(QLatin1String("INSERT INTO itemlist (itemId, desktopName, name, iconName, type, placeholder, categories) "
                                        "VALUES (?, ?, ?, ?, ?, ?, ?)")))
        qDebug() << sqlQuery.lastError().text();
    sqlQuery.addBindValue(itemId);
    sqlQuery.addBindValue(desktopName);
    sqlQuery.addBindValue(name);
    sqlQuery.addBindValue(iconName);
    sqlQuery.addBindValue(type);
    sqlQuery.addBindValue(placeholder);
    sqlQuery.addBindValue(categories);
    sqlQuery.exec();
    return sqlQuery.lastInsertId();
}

void dataBaseOperation::modifyItemData(quint32 id, const QString &desktopName, const QString &name, const QString &iconName, const QList<int> &_categories)
{
    QString categories = UKUI::fromQListIntToString(_categories);
    QSqlQuery sqlQuery;
    sqlQuery.prepare("UPDATE itemlist SET desktopName=?, name=?, iconName=?, categories=? WHERE itemId=?");
    sqlQuery.addBindValue(desktopName);
    sqlQuery.addBindValue(name);
    sqlQuery.addBindValue(iconName);
    sqlQuery.addBindValue(categories);
    sqlQuery.addBindValue(id);
    if(!sqlQuery.exec())
        qDebug() << sqlQuery.lastError();
}

QVariant dataBaseOperation::addFlipSet(quint32 setId, const QList<QList<quint32>> &sequence)
{
    QString itemSequence;
    if(!sequence.isEmpty()) {
        itemSequence += UKUI::fromQListUIntToString(sequence.first());
        for(int i = 1; i < sequence.size(); i++) {
            QList<quint32> setPageItemSeq = sequence.at(i);
            itemSequence += QString(";") += UKUI::fromQListUIntToString(setPageItemSeq);
        }
    }
    qDebug() << __func__ << itemSequence;

    QSqlQuery sqlQuery;
    if (!sqlQuery.prepare(QLatin1String("INSERT INTO sets (setId, sequence, scrollable) VALUES (?, ?, ?)")))
        qDebug() << sqlQuery.lastError().text();
    sqlQuery.addBindValue(setId);
    sqlQuery.addBindValue(itemSequence);
    sqlQuery.addBindValue(0);
    sqlQuery.exec();
    return sqlQuery.lastInsertId();
}

QVariant dataBaseOperation::addScrollSet(quint32 setId, const QList<quint32> &sequence)
{
    QString itemSequence;
    if(!sequence.isEmpty())
        itemSequence = UKUI::fromQListUIntToString(sequence);

    QSqlQuery sqlQuery;
    if (!sqlQuery.prepare(QLatin1String("INSERT INTO sets (setId, sequence, scrollable) VALUES (?, ?, ?)")))
        qDebug() << sqlQuery.lastError().text();
    sqlQuery.addBindValue(setId);
    sqlQuery.addBindValue(itemSequence);
    sqlQuery.addBindValue(1);
    sqlQuery.exec();
    return sqlQuery.lastInsertId();
}

void dataBaseOperation::modifyFlipSetData(quint32 setId, const QList<QList<quint32>> &sequence)
{
    QString itemSequence;
    if(!sequence.isEmpty()) {
        itemSequence += UKUI::fromQListUIntToString(sequence.first());
        for(int i = 1; i < sequence.size(); i++) {
            QList<quint32> setPageItemSeq = sequence.at(i);
            itemSequence += QString(";") += UKUI::fromQListUIntToString(setPageItemSeq);
        }
    }
    qDebug() << __func__ << "itemSequence:" << itemSequence;

    QSqlQuery sqlQuery;
    sqlQuery.prepare("UPDATE sets SET sequence=? WHERE setId=?");
    sqlQuery.addBindValue(itemSequence);
    sqlQuery.addBindValue(setId);
    if(!sqlQuery.exec())
        qDebug() << __func__ << __LINE__ << sqlQuery.lastError();
}

void dataBaseOperation::modifyScrollSetData(quint32 setId, const QList<quint32> &sequence)
{
    QString itemSequence;
    if(!sequence.isEmpty())
        itemSequence = UKUI::fromQListUIntToString(sequence);
    qDebug() << __func__ << itemSequence << setId;
    QSqlQuery sqlQuery;
    sqlQuery.prepare("UPDATE sets SET sequence=? WHERE setId=?");
    sqlQuery.addBindValue(itemSequence);
    sqlQuery.addBindValue(setId);
    if(!sqlQuery.exec())
        qDebug() << __func__ << __LINE__ << sqlQuery.lastError();
}

bool dataBaseOperation::modifySetName(quint32 setId, QString name)
{
    if(name.isEmpty())
        return false;
    QSqlQuery sqlQuery;
    sqlQuery.prepare("UPDATE itemlist SET name=? WHERE setId=?");
    sqlQuery.addBindValue(name);
    sqlQuery.addBindValue(setId);
    if(!sqlQuery.exec()) {
        qDebug() << sqlQuery.lastError();
        return false;
    }
    return true;
}

void dataBaseOperation::addPage(quint32 pageId, const QList<quint32> &sequence)
{
    QString itemSequence = UKUI::fromQListUIntToString(sequence);

    QSqlQuery sqlQuery;
    if (!sqlQuery.prepare(QLatin1String("INSERT INTO page (pageId, sequence) VALUES (?, ?)")))
        qDebug() << sqlQuery.lastError().text();
    sqlQuery.addBindValue(pageId);
    sqlQuery.addBindValue(itemSequence);
    sqlQuery.exec();
//    return sqlQuery.lastInsertId();
}

void dataBaseOperation::modifyPageData(quint32 pageId, const QList<quint32> &sequence)
{
    QString itemSequence = UKUI::fromQListUIntToString(sequence);
    qDebug() << __func__ << itemSequence << pageId;
    QSqlQuery sqlQuery;
    sqlQuery.prepare("UPDATE page SET sequence=? WHERE pageId=?");
    sqlQuery.addBindValue(itemSequence);
    sqlQuery.addBindValue(pageId);
    if(!sqlQuery.exec())
        qDebug() << __func__ << __LINE__ << sqlQuery.lastError();
}

void dataBaseOperation::deleteItemListData(int id)
{
    if(isTableExist("itemlist")) {
        QSqlQuery sqlQuery;
        if(!sqlQuery.exec(QString("DELETE FROM itemlist WHERE itemId = %1").arg(id)))
            qDebug() << sqlQuery.lastError();
        else
            qDebug() << "Delete ItemList Data success ! ! !" << "Id:" << id;
    }
}

void dataBaseOperation::deletePageData(int id)
{
    if (id < 0) {
        qDebug() << "param id error" << id;
        return;
    }
    if(isTableExist("page"))
    {
        QSqlQuery sqlQuery;
        if(!sqlQuery.exec(QString("DELETE FROM page WHERE pageId = %1").arg(id)))
            qDebug() << sqlQuery.lastError();
        else
            qDebug() << "Delete Page Data success ! ! !" << "PageId:" << id;
    }
    else
        return;
}

void dataBaseOperation::deleteSetsData(int id)
{
    if(isTableExist("sets")) {
        QSqlQuery sqlQuery;
        if(!sqlQuery.exec(QString("DELETE FROM sets WHERE setId = %1").arg(id)))
            qDebug() << sqlQuery.lastError();
        else
            qDebug() << "Delete Data success ! ! !";
    }
    else
        return;
}

void dataBaseOperation::deleteTable(const QString &tableName)
{
    QSqlQuery sqlQuery;
    if(!sqlQuery.exec(QString("DROP TABLE %1").arg(tableName)))
        qDebug() << sqlQuery.lastError();
    else
        qDebug() << "Delete Table success ! ! !";
}

void dataBaseOperation::closeDataBase()
{
    db.close();
}
